#include <ESP32BleGamepad.h>
#include <NintendoExtensionCtrl.h>

BleGamepad bleGamepad;
ClassicController classic;

void setup()
{
  Serial.begin(115200);
  bleGamepad.begin(false);

  classic.begin();
  while (!classic.connect())
  {
    Serial.println("連上Classic控制器，開始啟動藍牙…");
    delay(1000);
  }
}

int rightStick(uint8_t input)
{
  if (input > 24)
    return 255;
  else if (input > 20)
    return 127;
  else if (input < 8)
    return 0;
  else if (input < 12)
    return 64;
  return 127;
}

int leftStick(uint8_t input)
{
  if (input > 45)
    return 255;
  else if (input > 35)
    return 191;
  else if (input < 15)
    return 0;
  else if (input < 25)
    return 64;
  return 127;
}

void loop()
{
  boolean success = classic.update();

  if (!success)
  {
    Serial.println("Classic控制器斷線了～重新連線…");
    classic.reconnect();
    delay(100);
  }
  else
  {
    // 顯示Classic控制器的按鍵狀態
    classic.printDebug();

    bleGamepad.setButton(2, classic.buttonA());
    bleGamepad.setButton(1, classic.buttonB());
    bleGamepad.setButton(3, classic.buttonX());
    bleGamepad.setButton(0, classic.buttonY());

    bleGamepad.setButton(4, classic.buttonL());
    bleGamepad.setButton(5, classic.buttonR());
    bleGamepad.setButton(6, classic.buttonZL());
    bleGamepad.setButton(7, classic.buttonZR());

    bleGamepad.setButton(9, classic.buttonPlus());
    bleGamepad.setButton(8, classic.buttonMinus());
    bleGamepad.setButton(12, classic.buttonHome());

    bleGamepad.setXAxis(leftStick(classic.leftJoyX()));
    bleGamepad.setYAxis(255 - leftStick(classic.leftJoyY()));
    bleGamepad.setZAxis(rightStick(classic.rightJoyX()));
    bleGamepad.setZAxisRotation(255 - rightStick(classic.rightJoyY()));

    if (classic.dpadUp())
    {
      if (classic.dpadLeft())
        bleGamepad.setHatSwitch(7);
      else if (classic.dpadRight())
        bleGamepad.setHatSwitch(1);
      else
        bleGamepad.setHatSwitch(0);
    }
    else if (classic.dpadDown())
    {
      if (classic.dpadLeft())
        bleGamepad.setHatSwitch(5);
      else if (classic.dpadRight())
        bleGamepad.setHatSwitch(3);
      else
        bleGamepad.setHatSwitch(4);
    }
    else if (classic.dpadRight())
      bleGamepad.setHatSwitch(2);
    else if (classic.dpadLeft())
      bleGamepad.setHatSwitch(6);
    else
      bleGamepad.setHatSwitch(-1);
  }

  bleGamepad.sendState();
  delay(1);
}
